@@ -26,12 +26,13 @@ RSA *amap_rsa = NULL;
 
 // HELP //
 void help(char *prg) {
-  printf("%s %s (c) %s by %s <%s> %s\n", AMAP_PROGRAM, AMAP_VERSION, AMAP_YEAR, AMAP_AUTHOR, AMAP_EMAIL, AMAP_RESOURCE);
-  printf("Syntax: %s [-A|-B|-P] [-1buSRHUdqv] [[-m] -o <file>] [-D <file>] [-t/-T sec] [-c cons] [-C retries] [-p proto] [-i <file>] [target port [port] ...]\n", prg);
+  printf("%s v%s (c) %s by %s <%s> %s\n", AMAP_PROGRAM, AMAP_VERSION, AMAP_YEAR, AMAP_AUTHOR, AMAP_EMAIL, AMAP_RESOURCE);
+  printf("Syntax: %s [-A|-B|-P|-W] [-1buSRHUdqv] [[-m] -o <file>] [-D <file>] [-t/-T sec] [-c cons] [-C retries] [-p proto] [-i <file>] [target port [port] ...]\n", prg);
   printf("Modes:\n");
   printf("  -A         Map applications: send triggers and analyse responses (default)\n");
   printf("  -B         Just grab banners, do not send triggers\n");
-  printf("  -P         No banner or application stuff - be a (full connect) port scanner!\n");
+  printf("  -P         No banner or application stuff - be a (full connect) port scanner\n");
+  printf("  -W         Web Update - online update the application fingerprint database!\n");
   printf("Options:\n");
   printf("  -1         Only send triggers to a port until 1st identification. Speeeeed!\n");
   printf("  -b         Print ascii banner of responses\n");
@@ -58,8 +59,9 @@ void help(char *prg) {
   printf("  TARGET PORT   The target address and port(s) to scan (additional to -i)\n");
   printf("%s is a tool to identify application protocols on target ports.\n", AMAP_PROGRAM);
 #ifndef OPENSSL
-  printf("Note: this version was NOT compiled with SSL support\n");
+  printf("Note: this version was NOT compiled with SSL support!\n");
 #endif
+  printf("Usage hint: Options \"-bqv\" are recommended, add \"-1\" for fast/rush checks.\n");
   exit(-1);
 }
 
@@ -119,6 +121,176 @@ void amap_warn(char *string, ...) {
 }
 
 
+// AMAP_GET_DATA_TOKEN - partial rip from vh-lib //
+char *amap_get_data_token(char *data, char token) {
+  static char vdata[AMAP_MAXTOKENLEN] = "";
+  char search[4] = "#X:";
+  char *ptr;
+
+  search[1] = token;
+  if (strncmp(data, "###", 3) != 0) {
+//    amap_warn("invalid or missing version data: %s", data);
+    return(vdata);
+  }
+  if ((ptr = strstr(data, search)) == NULL) {
+//    amap_warn("missing token in data: %s", data);
+    return(vdata);
+  }
+  
+  memcpy(vdata, ptr + 3, sizeof(vdata)-1);
+  vdata[sizeof(vdata) - 1] = 0;
+  if ((ptr = index(vdata, '#')) == NULL) {
+//    amap_error("invalid or missing version string in webfile: %s", data);
+    strcpy(vdata, "");
+    return(vdata);
+  }
+  *ptr = 0;
+  return(vdata);
+}
+
+
+// AMAP_WEBUPDATE_FILE - partial rip from vh-lib //
+int amap_webupdate_file(char *webfile, char *localfile, int checkversion, int ask) {
+  int len = strlen("http://");
+  int wlen = strlen(webfile);
+  int port = 80, s, result = 1, datalen = 0, version = -1, newversion;
+  unsigned long int ip;
+  time_t epoch;
+  struct in_addr in;
+  struct hostent *target;
+  struct sockaddr_in addr;
+  struct tm *the_time;
+  char *url, *ptr, *data = NULL, *filedata;
+  char *host = malloc(strlen(webfile));
+  char *request = malloc(AMAP_WEBBUFLEN + wlen);
+  char datetime[64] = "";
+  FILE *f;
+
+  if (strncmp(webfile, "http://", len) != 0)
+    amap_error("webfile location is missing http://: %s", webfile);
+  if ((url = index(webfile + len, '/')) == NULL)
+    amap_error("webfile definition is missing a web file location: %s", webfile);
+  memset(host, 0, wlen);
+  memset(request, 0, AMAP_WEBBUFLEN + wlen);
+  memcpy(host, webfile + len, url - (webfile + len));
+  if (index(host, '@') != NULL)
+    amap_error("authentication not supported: %s", host);
+  if ((ptr = index(host, ':')) != NULL) {
+    *ptr++ = 0;
+    port = atoi(ptr);
+    if (port < 1 || port > 65535)
+      amap_error("invalid port: %s", ptr);
+  }
+  snprintf(request, AMAP_WEBBUFLEN + wlen, "GET %s HTTP/1.0\r\nHost: %s:%d\r\nUser-Agent: %s %s\r\n\r\n", url, host, port, AMAP_PROGRAM, AMAP_VERSION);
+
+#ifndef CYGWIN
+  if (inet_pton(AF_INET, host, &in) <= 0) {
+#else
+  if (inet_aton(host, &in) <= 0) {
+#endif
+    if ((target = gethostbyname(host)) != NULL)
+      memcpy((char*)&ip, (char*)target->h_addr, 4);
+    else
+      amap_error("could not resolve host: %s", host);
+  } else
+    memcpy((char*)&ip, (char*)&in.s_addr, 4);
+  if ((s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
+    amap_error("could not get a socket");
+  setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &result, sizeof(result));
+  addr.sin_port = htons(port);
+  addr.sin_family = AF_INET;
+  memcpy(&addr.sin_addr.s_addr, &ip, 4);
+  if (connect(s, (struct sockaddr *) &addr, sizeof(addr)) < 0)
+    amap_error("could not connect to host: %s", host);
+
+  if (send(s, request, strlen(request), 0) < 0)
+    amap_error("web site closed connection");
+  memset(request, 0, AMAP_WEBBUFLEN + wlen);
+  
+  while ((len = recv(s, request, AMAP_WEBBUFLEN, 0)) > 0) {
+    if (data == NULL)
+      data = malloc(len);
+    else
+      data = realloc(data, datalen + len);
+    memcpy(data + datalen, request, len);
+    datalen += len;
+    memset(request, 0, AMAP_WEBBUFLEN);
+  }
+  close(s);
+
+  // prevent memory access voilation if no version data is present
+  data = realloc(data, datalen + AMAP_MAXTOKENLEN);
+  memset(data + datalen, 0, AMAP_MAXTOKENLEN);
+
+  if (strncmp(data, "HTTP/", strlen("HTTP/")) != 0)
+    amap_error("invalid http response: %s", data);
+  if (strncmp(data + strlen("HTTP/1.0 "), "200", 3) != 0)
+    amap_error("file could not be found by web server: %s", data);
+  if ((filedata = strstr(data, "\r\n\r\n")) == NULL)
+    amap_error("no data found in response: %s", data);
+  filedata += 4;
+  datalen = datalen - (filedata - data);
+
+  // versioncheck > 0 - check version information on file
+  // format: "###V:104#P:1.0#D:1231231231#M:Have fun!###DO NOT EDIT THIS LINE!"
+  if (checkversion) {
+    if ((f = fopen(localfile, "r")) != NULL) {
+      memset(request, 0, AMAP_WEBBUFLEN);
+      fread(request, AMAP_WEBBUFLEN - AMAP_MAXTOKENLEN, 1, f);
+      version = atoi(amap_get_data_token(request, 'V'));
+      fclose(f);
+    }
+    
+    if (version >= atoi(amap_get_data_token(filedata, 'V'))) {
+      printf("No new updates for file %s available\n", localfile);
+      free(request);
+      free(host);
+      free(data);
+      return -1;
+    }
+  }
+
+  // if ask > 0 - ask if overwrite file
+  if (ask) {
+    printf("Please confirm updating of file %s [YES(default)/no]: ", localfile);
+    result = fgetc(stdin);
+    if (result == 'N' || result == 'n')
+      return -1;
+  }
+  
+  // get other data
+  if ((epoch = strtoul(amap_get_data_token(filedata, 'D'), NULL, 10)) > 1000000000) {
+    the_time = localtime(&epoch);
+    strftime(datetime, sizeof(datetime), " (data from %Y-%m-%d %H:%M:%S)", the_time);
+  }
+  
+  // write file
+  if ((f = fopen(localfile, "w")) == NULL)
+    amap_error("can not write file %s", localfile);
+  fwrite(filedata, datalen, 1, f);
+  fclose(f);
+  printf("File %s successfully updated%s\n", localfile, datetime);
+
+  // msg check
+  ptr = amap_get_data_token(filedata, 'M');
+  if (strlen(ptr) > 0) {
+    printf("This update comes with the following message:\n\"%s\"\n", ptr);
+  }
+
+  // main program version check
+  ptr = amap_get_data_token(filedata, 'P');
+  if (strlen(ptr) > 2) {
+    if (strcmp(ptr, AMAP_VERSION) != 0)
+      printf("A new version of %s is available! You are using v%s, current is v%s.\nGo and download from %s !\n", AMAP_PROGRAM, AMAP_VERSION, ptr, AMAP_RESOURCE);
+  }
+
+  free(request);
+  free(host);
+  free(data);
+  return 0;
+}
+
+
 #ifdef OPENSSL
 // AMAP_SSL_TEMP_RSA_CB //
 RSA *amap_ssl_temp_rsa_cb(SSL *ssl, int export, int keylength) {
@@ -1722,7 +1894,7 @@ int main(int argc, char *argv[]) {
   // GETOPT //
   if (argc < 2 || strncmp(argv[1], "-?", 2) == 0 || strncmp(argv[1], "--h", 3) == 0)
     help(argv[0]);
-  while ((i = getopt(argc, argv, "1SRHUbuvdhmi:T:c:C:p:o:D:t:qABP")) >= 0) {
+  while ((i = getopt(argc, argv, "1SRHUbuvdhmi:T:c:C:p:o:D:t:qABPW")) >= 0) {
     switch (i) {
     case 'A': // defines in the future that we want default AMAP mode
       break;
@@ -1796,6 +1968,9 @@ int main(int argc, char *argv[]) {
       opt.one_is_enough = 1;
       opt.timeout_response--;
       break;
+    case 'W':
+      opt.update = 1;
+      break;
     case 'h':
       help(argv[0]);
       break;
@@ -1806,7 +1981,9 @@ int main(int argc, char *argv[]) {
   }
 
   // VARIABLES VERIFICATION //
-  if ((optind + 2 > argc) && (file_nmap == NULL))
+  if (opt.update && argc != 2 && filename == NULL)
+    amap_warn("amap takes no other commandline options when in -W online update mode except -D");
+  if ((optind + 2 > argc) && (file_nmap == NULL) && (opt.update == 0))
     help(argv[0]);
   if (file_log == NULL && opt.machine_readable)
     amap_error("option -m set, but no logfile defined (-o)");
@@ -1816,6 +1993,41 @@ int main(int argc, char *argv[]) {
     amap_error("the connect timeout option (-T) must be between 1 and 240, its counted in seconds!");
   if (opt.timeout_response < 1 || opt.timeout_response > 240)
     amap_error("the response timeout option (-t) must be between 1 and 240, its counted in seconds!");
+
+  // ONLINE UPDATE //
+  if (opt.update) {
+    printf("Running Online Update for fingerprints, connecting to %s\n", AMAP_RESOURCE);
+    if (filename == NULL) {
+      filename = malloc(strlen(AMAP_APPDEF_PATH) + strlen(AMAP_DEFAULT_FILENAME) + 2);
+      strcpy(filename, AMAP_APPDEF_PATH);
+      strcat(filename, "/");
+      strcat(filename, AMAP_DEFAULT_FILENAME);
+    }
+    file_log = malloc(strlen(filename) + 6);
+    strcpy(file_log, filename);
+    file_nmap = malloc(strlen(AMAP_RESOURCE) + strlen(file_log) + 16);
+    strcpy(file_nmap, "http://");
+    strcat(file_nmap, AMAP_RESOURCE);
+    strcat(file_nmap, "/");
+    strcat(file_nmap, AMAP_DEFAULT_FILENAME);
+    strcat(file_log, AMAP_FILETYPE_RESPONSES);
+    ptr = malloc(strlen(file_nmap) + 8);
+    strcpy(ptr, file_nmap);
+    strcat(ptr, AMAP_FILETYPE_RESPONSES);
+    amap_webupdate_file(ptr, file_log, 1, 0);
+    strcpy(file_log, filename);
+    strcat(file_log, AMAP_FILETYPE_TRIGGERS);
+    strcpy(ptr, file_nmap);
+    strcat(ptr, AMAP_FILETYPE_TRIGGERS);
+    amap_webupdate_file(ptr, file_log, 1, 0);
+    strcpy(file_log, filename);
+    strcat(file_log, AMAP_FILETYPE_RPC);
+    strcpy(ptr, file_nmap);
+    strcat(ptr, AMAP_FILETYPE_RPC);
+    amap_webupdate_file(ptr, file_log, 1, 0);
+    printf("Done with Online Update.\n");
+    exit(0);
+  }
   
   // READING FILES //
   if (file_nmap != NULL)
@@ -1983,13 +2195,13 @@ int main(int argc, char *argv[]) {
     (void) setvbuf(opt.logfile, NULL, _IONBF, 0);
     if (opt.machine_readable)
       fprintf(opt.logfile, "# ");
-    fprintf(opt.logfile, "%s %s (%s) started at %s - ", AMAP_PROGRAM, AMAP_VERSION, AMAP_RESOURCE, today);
+    fprintf(opt.logfile, "%s v%s (%s) started at %s - ", AMAP_PROGRAM, AMAP_VERSION, AMAP_RESOURCE, today);
     if (opt.portscanner)
       fprintf(opt.logfile, "PORTSCAN mode\n");
     else if (opt.banner_only)
-      fprintf(opt.logfile, "BANNER GRAB mode\n");
+      fprintf(opt.logfile, "BANNER mode\n");
     else
-      fprintf(opt.logfile, "APPLICATION MAP mode\n");
+      fprintf(opt.logfile, "MAPPING mode\n");
     if (opt.machine_readable)
       fprintf(opt.logfile, "# IP_ADDRESS:PORT:PROTOCOL:PORT_STATUS:SSL:IDENTIFICATION:PRINTABLE_BANNER:FULL_BANNER\n");
   }
@@ -1997,13 +2209,13 @@ int main(int argc, char *argv[]) {
   // STARTING SCAN //
   if (opt.verbose)
     printf("\n");
-  printf("%s %s (%s) started at %s - ", AMAP_PROGRAM, AMAP_VERSION, AMAP_RESOURCE, today);
+  printf("%s v%s (%s) started at %s - ", AMAP_PROGRAM, AMAP_VERSION, AMAP_RESOURCE, today);
   if (opt.portscanner)
     printf("PORTSCAN mode\n\n");
   else if (opt.banner_only)
-    printf("BANNER GRAB mode\n\n");
+    printf("BANNER mode\n\n");
   else
-    printf("APPLICATION MAP mode\n\n");
+    printf("MAPPING mode\n\n");
   
   if (amap_scan(AMAP_SCANMODE_DEFAULT, targets, triggers, responses, &opt) == 0)
     amap_error("nothing to scan - no open ports in nmap output file");
@@ -2015,7 +2227,7 @@ int main(int argc, char *argv[]) {
     (void) amap_scan(AMAP_SCANMODE_RPC, targets, triggers_rpc, NULL, &opt);
 
   // PRINTING UNIDENTIFIED PORTS //
-  if (opt.portscanner == 0 && opt.banner_only == 0) {
+  if (opt.portscanner == 0 && opt.banner_only == 0 && opt.quiet == 0) {
     i = 0;
     target_tmp = targets;
     printf("\nUnidentified ports:");
@@ -2087,9 +2299,9 @@ int main(int argc, char *argv[]) {
   if (opt.logfile != NULL) {
     if (opt.machine_readable)
       fprintf(opt.logfile, "# ");
-    fprintf(opt.logfile, "%s %s finished at %s\n", AMAP_PROGRAM, AMAP_VERSION, today);
+    fprintf(opt.logfile, "%s v%s finished at %s\n", AMAP_PROGRAM, AMAP_VERSION, today);
   }
-  printf("\n%s %s finished at %s\n", AMAP_PROGRAM, AMAP_VERSION, today);
+  printf("\n%s v%s finished at %s\n", AMAP_PROGRAM, AMAP_VERSION, today);
 
   return 0;
 }
