@@ -18,11 +18,11 @@
  */
 
 #define AMAP_PROGRAM   "amap"
-#define AMAP_VERSION   "v4.7"
-#define AMAP_YEAR      "2004"
+#define AMAP_VERSION   "4.8"
+#define AMAP_YEAR      "2005"
 #define AMAP_AUTHOR    "van Hauser and DJ RevMoon"
 #define AMAP_EMAIL     "amap-dev@thc.org"
-#define AMAP_RESOURCE  "www.thc.org"
+#define AMAP_RESOURCE  "www.thc.org/thc-amap"
 
 #ifndef AMAP_PREFIX
  #ifdef PREFIX
@@ -33,11 +33,15 @@
  #endif
 #endif
 
-#define AMAP_APPDEF_PATH	AMAP_PREFIX"/bin"
+#define AMAP_APPDEF_PATH	AMAP_PREFIX"/etc"
 #define AMAP_BUFSIZE		1024	// standard buffer size
 #define AMAP_BUFSIZE_BIG	65536   // big standard buffer size
 #define AMAP_REGEX_OPTIONS	( PCRE_MULTILINE | PCRE_CASELESS | PCRE_DOTALL )
 
+/* web update feature */
+#define AMAP_WEBBUFLEN 1024
+#define AMAP_MAXTOKENLEN 64
+
 /* connection and task definitions */
 #define AMAP_MAX_CONNECT_RETRIES	3	// connect() retries
 #define AMAP_CONNECT_TIME	5	// seconds to wait for connect
@@ -86,6 +90,7 @@ typedef struct {
   char  banner;
   char  banner_only;
   char  portscanner;
+  char  update;
   char  machine_readable;
   char  harmful;
   char  one_is_enough;
@@ -157,7 +162,7 @@ typedef struct {
   char active;
   char ssl_enabled;
   char retry;
-  unsigned char response[AMAP_BUFSIZE];
+  unsigned char response[AMAP_BUFSIZE + 1];
   int socket;
   int response_length;
   int sockaddr_len;
